/* 
 * CSE142, 98 Autumn
 * Homework 4
 * Weather Gods and Goddesses

  Part D:

  Author: Zsolt Mathe
  Sect: BH   CSE 142
  ID: 9833991
  mathez@u.washington.edu

  Date: 11/10/98

*/


/* Program Description:
    Draw a landscape of trees, in which the viewer can move freely
    using the keypad or the mouse. Also allow selection of various
    types of precipitation and their color.


    coordinate system used:
    x = left - right
    y = up - down
    z = in - out
*/


#include "gp142.h"
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <memory.h>
#include <time.h>



#ifndef TRUE	
#define TRUE    		    1
#endif
#ifndef FALSE
#define FALSE   		    0
#endif

/* randomize number +/- 30% */
#define RAND30(a)		    (a + (rand() % ((a/3)+2))-(a/3))


#define PI				    3.14159265359


/* For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.
 */
#define FILL 0


#define BUTTON_WIDTH		80
#define BUTTON_HEIGHT		20
#define B_TITLE_SIZE		20	/* size of button title string */
#define MAX_BUTTONS			20  /* max number of buttons supported*/
#define NO_BUTTON_HIT		-1

#define INFO_BAR_HEIGHT		60  /* height of bar at the bottom
								of screen where the program info
								is printed*/


/* Types of precipitations we support 
these are also button ids (start from 1000 so they
dont confilct with color buttons)*/
#define NONE		        1000
#define RAIN		        1001
#define SNOW		        1002
#define BALLS		        1003
#define HAIL		        1004

/* preciptiation speeds */
#define RAIN_SPEED		    7
#define SNOW_SPEED		    3
#define BALL_SPEED		    4
#define BALL_H_SPEED        10   /* horizontal speed for balls only */
#define HAIL_SPEED		    10

/* their size */
#define RAIN_SIZE			7
#define SNOW_SIZE			4
#define BALL_SIZE			10
#define HAIL_SIZE			4

/* their color */
#define RAIN_COLOR			LT_GRAY
#define SNOW_COLOR			WHITE
#define BALL_COLOR			RED
#define HAIL_COLOR			WHITE

/* and their qty (must be less or equal than MAX_PRECIP) */
#define RAIN_QTY		    300
#define SNOW_QTY		    200
#define BALL_QTY		    80
#define HAIL_QTY		    100


/* types of commands we support
also button ids*/
#define GO_LEFT		        1005
#define GO_RIGHT	        1006
#define LIGHTEN_UP	        1007
#define FAST		        1008


#define SLOW_SPEED			15.0
#define FAST_SPEED			40.0
#define TURNING_ANGLE		0.06		/* turning angle increment 
										in radians */
#define MAX_OBJECTS			500			/* max number of objects
										in world */

#define TREE_OBJ			0			/* object types */
#define TRIANGLE_OBJ		1

#define MAX_TRIANGLES		40			/* max number of triangles
										in an POLY object */
#define TRI_EDGES			3			/* always 3 edges in a triangle*/

#define TRI_E1				0			/* edge indexes in the triangle */
#define TRI_E2				1			
#define TRI_E3				2			




#define MAX_PRECIP			300			/* max number of drops */
#define NUM_TREES			400
#define WORLD_BOUNDS_X		2000		/* -X to X */
#define WORLD_BOUNDS_Z		12000		/* -Z to Z */
#define VIEWING_DISTANCE	1000.0		/* used in perspective projection*/


#define Y_GROUND			-40.0		/* ground height */
#define Y_FLY				-500.0		/* flying height */


#define TEX_SIZE			128			/* ground texture size 
										 must be power of 2 */
#define TRANSP_COLOR		127			/* texture transparent color*/



/* structs */

typedef struct {
	double	x,y,z;
} Point3;


typedef struct {
	int			color;
	int			size;
} Tree;



typedef struct {
	int			color;			/* triangle color */
	
	Point3		e[TRI_EDGES];	/* triangle edges */
	Point3		te[TRI_EDGES]; /* transformed edges */
} Triangle;



typedef struct {
	int			num_triangles;
	Triangle	tri[MAX_TRIANGLES];		/* triangle array */
	Triangle	*sort_tri[MAX_TRIANGLES];  /* sorted triangle pointer
										   array */
} TriangleList;



typedef struct {
	int			obj_type;		/* object type */
	Point3		o;				/* object center point */
	Point3		to;				/* transformed (rotated+translated)
								center point*/
	union {
		Tree			tree;	/* object is either a tree or */
		TriangleList	*tri_list;  /* a triangle object */
	};

} Object;


typedef struct {
	GP142_point	precip;			/* drop coordinates */
	int			x_speed;			/* its speed (incremental value) */
    int         y_speed;
	int			size;			/* size (either radius, width or length)*/
} Precip;


typedef struct {
	int		num_objects;		/* number of objects in world */
	Object	obj[MAX_OBJECTS];

	Point3	pos;				/* current view position and angle*/
	double	angle;

	int		fast;				/* are drops falling fast? */
	int		precip_type;		/* current precipitiation */
	int		precip_color;		/* current precip color */
	int		precip_qty;			/* current qty of drops */
	Precip	drops[MAX_PRECIP];	/* array that holds precipitation data*/

	char	texture[TEX_SIZE*TEX_SIZE]; /* holds ground texture*/

	int		splash;				/* TRUE if splash screen is displayed*/
} World;



typedef struct {
	GP142_point		ll_pos;					/* lower left corner */
	char			title[B_TITLE_SIZE];	/* button title */
	int				button_id;				/* unique button id (positive)*/
	int				hilited;				/* TRUE/FALSE */
} ButtonData;


typedef struct {
	int				num_buttons;
	ButtonData		buttons[MAX_BUTTONS];
} ButtonList;




/* function prototypes */


/* general */

/* setup all buttons needed */
void AddAllButtons(ButtonList *bl);

/* print info at bottom of screen */
void DrawInfo(void);

/* draw initial splash screen */
void DrawSplash(void);



/* event handling/dispatch functions */
void HandleKbdEvent(World *w, GP142_event_t *event_t);
void HandleMouseEvent(World *w, ButtonList *bl, GP142_event_t *event_t);
void HandlePerEvent(World *w);



/* button handling */
/* initialize use of buttons once at the beginning of program */
void ButtonInit(ButtonList *bl);
/* add a button to the list*/
void AddButton(ButtonList *bl, GP142_point *ll_pos, char *title, int id);
/* check if a button was hit and return button id or NO_BUTTON_HIT */
int ButtonHit(ButtonList *bl, GP142_point *mouse);
/* set a button to hilted or normal */
void SetButtonHilite(ButtonList *bl, int id, int hilite);
/* draw all buttons */
void DrawButtons(ButtonList *bl);






/* world handling */

World* InitWorld(void);

/* place objects objects in world*/
void GenerateWorld(World *w);

/* generate house in world */
void GenerateHouse(World *w);

/* generate ground texture (called from GenerateWorld)*/
void GenerateTexture(World *w);

/* free up memory allocated for world*/
void CloseWorld(World *w);

/* add a tree to the object list, its parameters and 
position are passed in tree, and pos respectively*/
void AddTreeObject(World *w, Tree *tree, Point3 *pos);

/* add a triangle list to the object list, again its
parameters and position are input parameters */
void AddTriObject(World *w, TriangleList *tri_list, Point3 *pos);

/* initialize triangle list */
void InitTriList(TriangleList *tri_list);

/* add triangle to triangle list */
void AddTriangle(TriangleList *tri_list, int color,
				 Point3 *e1, Point3 *e2, Point3 *e3);

/* add a rectangle to the triangle list (in fact
2 triangles) */
void AddRectangle(TriangleList *tri_list, int color,
				  Point3 *e1, Point3 *e2, Point3 *e3, Point3 *e4);

/* place viewer at a certain place in the world 
and transform the object array (rotating + translating)
the position is given by a position vector (incremental) 
 */
void SetView(World *w, double pos_inc, double angle_inc);


/* convert 3d coordinates to screen coordinates through perspective
projection.
mulz is a multiplication factor to size objects with.
function returns TRUE if point falls into the viewing region,
FALSE otherwise */

int ProjectPt(Point3 *pt, GP142_point *sp, double *mulz);

/* determines if triangle is visible from view point; returns
TRUE if visible*/
int IsTriangleVisible(Point3 *e1, Point3 *e2, Point3 *e3);

/* determine if triangle is visible on screen */
int IsTriangleOnScreen(GP142_point *p1, GP142_point *p2, 
					   GP142_point *p3);

/* determine drawing ourder of triangles */
int IsTriangleDrawnLast(Point3 *e1, Point3 *e2, Point3 *e3);

/* main function that draws projected objects and background */
void DrawScene(World *w);

/* draws a triangle object (a list of triangles) (called by
DrawScene). */
void DrawTriangleObject(TriangleList *tri_list);

/* project and draw triangle */
void DrawTriangle(int color, Point3 *e1, Point3 *e2, Point3 *e3);

/* draws just the background (called by DrawScene)*/
void DrawBackground(World *w);

/* maps the ground texture (called by DrawScene)*/
void DrawGround(World *w);

/* sort world object by distance so that further objects are drawn
first and closer ones last (called by DrawScene)*/
void SortObjects(World *w);


/* draw a single tree at position sp with size determined by mulz
(called by DrawScene) */
void DrawTree(Tree *tree, GP142_point *sp, double mulz);






/* precipitation functions */

/* set all drops to the top of screen, set their color, size and speed*/
void InitPrecip(World *w, ButtonList *bl, int button_id);

/* draw all precipitation */
void DrawPrecip(World *w);

/* make precipitation fall */
void PrecipFall(World *w);

/* draw various falling things */
void DrawRain(World *w);
void DrawSnow(World *w);		
void DrawBalls(World *w);		
void DrawHail(World *w);




/* misc functions */

/* print failure error message */
void BadError(void);




int main(void)
{
  int				quit;
  GP142_event_t		event_t;
  int				event;

  ButtonList		bl;			/* list that holds all buttons */
  World				*w;
  time_t t;			/*for random num gen init*/
  

  /*
   * INITIALIZATIONS
   */
      
  srand (time(&t));	      /* Seed the random number generator             */

  GP142_open();           /* Open and init the GP142 Graphics Window      */
  GP142_logging(LOG_OFF); 

  GP142_clear();	      /* This will clear the window & paint it white. */
  GP142_animate(ANI_RUN); /* Start animation so precips will grow  */

#ifdef WINDOWS
  GP142_show_text(FALSE); /* hide text window (for Windows only)*/
#endif

  ButtonInit(&bl);
  AddAllButtons(&bl);

  w=InitWorld();
  GenerateWorld(w);

  /*
   * The main event loop:
   * --- ---- ----- ----
   */
  quit=FALSE;
  while (!quit) {

	/*find out what has happened */
	event = GP142_await_eventP(&event_t);

	switch (event) {
    case GP142_QUIT:
      quit = TRUE;           /* set flag to terminate loop           */
      break;

    case GP142_KBD:
		HandleKbdEvent(w,&event_t);
      break;

    case GP142_MOUSE:
		HandleMouseEvent(w,&bl,&event_t);
      break;

    case GP142_PERIODIC:
		HandlePerEvent(w);
      break;

    default:

      break;
	
    }  /*end switch */

	if(w->splash)
	{	
		/* draw splash screen */
		DrawSplash();
	} else
	{
		/* draw trees, ground and mountains */
		DrawScene(w);
		/* draw the current precipitation (if any) */
		DrawPrecip(w);
		/* draw info bar at the bottom of screen */
		DrawInfo();
		/* draw all buttons */
		DrawButtons(&bl);
	}
   
  }  /* end event loop */

  
  /*
   * WRAP-UP
   */
  GP142_close();         /* clean up and close graphics window   */
  CloseWorld(w);

  return 0;
}  /*end main */

/*****************************************************************************

     FUNCTIONS

*****************************************************************************/


/* event handling / dispatch functions */

void HandleKbdEvent(World *w, GP142_event_t *event_t)
{
	
	if(w->splash==FALSE)
	{
		if(event_t->c=='4') SetView(w,0.0,-TURNING_ANGLE); /* left */
		if(event_t->c=='6') SetView(w,0.0,TURNING_ANGLE); /* right */
		if(event_t->c=='2') SetView(w,FAST_SPEED,0.0);  /* back */
		if(event_t->c=='8') SetView(w,-FAST_SPEED,0.0); /* ahead */

		PrecipFall(w);
	}

	w->splash=FALSE;
}



void HandleMouseEvent(World *w, ButtonList *bl, GP142_event_t *event_t)
{
	GP142_point		mouse;
	int				button_id;	/* button clicked */

	if(w->splash==FALSE)
	{
		mouse.x=event_t->x;
		mouse.y=event_t->y;
		button_id = ButtonHit(bl,&mouse);

		/* button was clicked */
		if(button_id!=NO_BUTTON_HIT)
		{
			switch(button_id)
			{
			case GO_LEFT:
				SetView(w,0.0,-TURNING_ANGLE);
				break;

			case GO_RIGHT:
				SetView(w,0.0,TURNING_ANGLE);
				break;

			case LIGHTEN_UP:
				if(w->precip_qty) w->precip_qty-=w->precip_qty/6;
				break;

			case FAST:
				if(w->fast)
				{
					SetButtonHilite(bl,FAST,FALSE);
					w->fast=FALSE;
				} else
				{
					SetButtonHilite(bl,FAST,TRUE);
					w->fast=TRUE;
				}
				break;

				/* precipitation buttons type or color were clicked*/
			default:
				InitPrecip(w,bl,button_id);
				break;
			}
				
		} else  /* no button clicked, rotate viewpoint toward click */
		{
				SetView(w,0.0,(double)mouse.x/VIEWING_DISTANCE);
		}
		PrecipFall(w);
	}

	w->splash=FALSE;
}



void HandlePerEvent(World *w)
{
	if(w->splash==FALSE)
	{
		/* viewpoint goes ahead slowly */
		SetView(w,-SLOW_SPEED,0.0);

		/* viewpoint falls to ground */
		if(w->pos.y<Y_GROUND)
			w->pos.y+=20;
		else
			w->pos.y=Y_GROUND;

		PrecipFall(w);
	}
}



/********************************************************************/

/* general functions */

/* draw information for user at the bottom of screen */
void DrawInfo(void)
{
	GP142_point		p1,p2;

	p1.x=-GP142_XMAX;
	p1.y=-GP142_YMAX;
	p2.x=GP142_XMAX;
	p2.y=-GP142_YMAX+INFO_BAR_HEIGHT;

	GP142_rectangleP(BLACK,p1,p2,FILL);

	p1.x=-GP142_XMAX+30;
	p1.y=-GP142_YMAX+6;
	GP142_printfP(WHITE,p1,10,
		"Select precipitation; movement: 4-left, 6-right, 8-ahead, 2-back");

	p1.x=-80;
	p1.y=-GP142_YMAX+30;
	GP142_printfP(YELLOW,p1,18,"The Weather Mobil");
}


/* draw splash screen */
void DrawSplash(void)
{
	GP142_point		p1,p2;

	p1.x=-GP142_XMAX;
	p1.y=GP142_YMAX;
	p2.x=GP142_XMAX;
	p2.y=-GP142_YMAX;
	GP142_rectangleP(BLACK,p1,p2,FILL);

	p1.x=-4*GP142_XMAX/5;
	p1.y=GP142_YMAX/2;
	p2.x=4*GP142_XMAX/5;
	p2.y=-GP142_YMAX/2;

	GP142_rectangleP(RED,p1,p2,5);

	p1.x=-100;
	p1.y=GP142_YMAX/3;
	GP142_printfP(MED_GRAY,p1,24,"The Weather Mobil");

	p1.x+=2;
	p1.y+=2;
	GP142_printfP(YELLOW,p1,24,"The Weather Mobil");

	p1.y-=30;
	p1.x=-90;
	GP142_printfP(YELLOW,p1,20,"Weather God/Godess");

	p1.x=(-4*GP142_XMAX/5)+30;
	p1.y=0;
	GP142_printfP(WHITE,p1,18,"Select a preciptitaion type on the left.");

	p1.y-=25;
	GP142_printfP(WHITE,p1,18,"Select a color on the right.");

	p1.y-=25;
	GP142_printfP(WHITE,p1,18,"To find the house, follow the Sun.");

	p1.y-=25;
	GP142_printfP(WHITE,p1,18,"Use 4-left, 6-right, 8-ahead, 2-back to move.");

	p1.y=(-GP142_YMAX/2)+20;
	GP142_printfP(WHITE,p1,14,"Author: Zsolt Mathe #9833991   Sect: BH  CSE 142");

	p1.y=-GP142_YMAX+20;
	p1.x=-GP142_XMAX+20;
	GP142_printfP(WHITE,p1,14,"Press any key or click mouse to continue");
}


#define BUTTON_GAP		3*BUTTON_HEIGHT/2

void AddAllButtons(ButtonList *bl)
{
	GP142_point		ll_pos;

	/* preciptiation type buttons */
	ll_pos.x=-GP142_XMAX+10;
	ll_pos.y=GP142_YMAX-BUTTON_HEIGHT*2;

	AddButton(bl,&ll_pos,"Rain",RAIN);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Snow",SNOW);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Balls",BALLS);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Hail",HAIL);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Lighten Up",LIGHTEN_UP);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"None",NONE);
	SetButtonHilite(bl,NONE,TRUE);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Fast!",FAST);

	/* color selection buttons */
	/* these can be any color, if they match #defined colors
	for precipitation then they will be hilited if that
	precipitation is selected */

	ll_pos.x=GP142_XMAX-10-BUTTON_WIDTH;
	ll_pos.y=GP142_YMAX-BUTTON_HEIGHT*2;
	
	AddButton(bl,&ll_pos,"White",WHITE);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Lt Gray",LT_GRAY);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Med Gray",MED_GRAY);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Red",RED);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Blue",BLUE);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Gold",GOLD);

	ll_pos.y-=BUTTON_GAP;
	AddButton(bl,&ll_pos,"Pink",PINK);

	/* direction buttons */

	ll_pos.y=-GP142_YMAX+30;
	ll_pos.x=-140-BUTTON_WIDTH;
	AddButton(bl,&ll_pos,"< LEFT <",GO_LEFT);

	ll_pos.x=140;
	AddButton(bl,&ll_pos,"> RIGHT >",GO_RIGHT);
}




/*******************************************************************/

/* precipitation functions */

/* place drops way up on the screen so that they will start
to fall from the top 
set their size, speed and color*/
void InitPrecip(World *w, ButtonList *bl, int button_id)
{
	int		i;
	int		precip_yspeed,precip_size;
    int     precip_xspeed;

	/* unhilite the last button clicked */
	SetButtonHilite(bl,w->precip_color,FALSE);
	SetButtonHilite(bl,w->precip_type,FALSE);

    precip_xspeed=0;    /* only balls have an x axis increment */

	switch(button_id)
	{
	case NONE:
		w->precip_color=BLACK;
		w->precip_type=NONE;
		precip_yspeed=0;
		precip_size=0;
		w->precip_qty=0;
		break;

	case RAIN:
		w->precip_color=RAIN_COLOR;
		w->precip_type=RAIN;
		precip_yspeed=RAIN_SPEED;
		precip_size=RAIN_SIZE;
		w->precip_qty=RAIN_QTY;
		break;

	case SNOW:
		w->precip_color=SNOW_COLOR;
		w->precip_type=SNOW;
		precip_yspeed=SNOW_SPEED;
		precip_size=SNOW_SIZE;
		w->precip_qty=SNOW_QTY;
		break;

	case BALLS:
		w->precip_color=BALL_COLOR;
		w->precip_type=BALLS;
		precip_yspeed=BALL_SPEED;
		precip_size=BALL_SIZE;
		w->precip_qty=BALL_QTY;
        precip_xspeed=((rand()%BALL_H_SPEED)+4);
        if(rand()%2) precip_xspeed=-precip_xspeed;
		break;

	case HAIL:
		w->precip_color=HAIL_COLOR;
		w->precip_type=HAIL;
		precip_yspeed=HAIL_SPEED;
		precip_size=HAIL_SIZE;
		w->precip_qty=HAIL_QTY;
		break;

	default:
		precip_yspeed=0;
		precip_size=0;
		w->precip_color=button_id;
		break;
	}

	/* hilite the newly clicked button */
	SetButtonHilite(bl,w->precip_type,TRUE);
	SetButtonHilite(bl,w->precip_color,TRUE);

	if(precip_yspeed)
	{
		for(i=0;i<w->precip_qty;i++)
		{
			w->drops[i].precip.x=(rand() % (GP142_XMAX*2)) - GP142_XMAX;
			w->drops[i].precip.y=(rand() % (GP142_YMAX*2)) + GP142_YMAX;

			w->drops[i].size=RAND30(precip_size);
			w->drops[i].y_speed=RAND30(precip_yspeed)+w->drops[i].size*2;
            w->drops[i].x_speed=precip_xspeed;
		}
	}
}

/* draw all precipitation defined by w->precip_type */

void DrawPrecip(World *w)
{
	switch(w->precip_type)
	{
	case RAIN:
		DrawRain(w);
		break;

	case SNOW:
		DrawSnow(w);
		break;

	case BALLS:
		DrawBalls(w);
		break;

	case HAIL:
		DrawHail(w);
		break;
	}
}

/* make the current precipitiation fall */

void PrecipFall(World *w)
{
	int		precip_size,precip_yspeed;
    int     precip_xspeed;
	int		i;

    precip_xspeed=0;

	switch(w->precip_type)
	{
	case RAIN:
		precip_size=RAIN_SIZE;
		precip_yspeed=RAIN_SPEED;
		break;

	case SNOW:
		precip_size=SNOW_SIZE;
		precip_yspeed=SNOW_SPEED;
		break;

	case BALLS:
		precip_size=BALL_SIZE;
		precip_yspeed=BALL_SPEED;
        precip_xspeed=((rand()%BALL_H_SPEED)+4);
        if(rand()%2) precip_xspeed=-precip_xspeed;
		break;

	case HAIL:
		precip_size=HAIL_SIZE;
		precip_yspeed=HAIL_SPEED;
		break;

	case NONE:
		w->precip_qty=0;
		break;
	}
	
	for(i=0;i<w->precip_qty;i++)
	{
		w->drops[i].precip.y-=w->drops[i].y_speed;
        w->drops[i].precip.x+=w->drops[i].x_speed;

		/* move snow flakes left-right */
		if(w->precip_type==SNOW)
			w->drops[i].precip.x+=(rand()%5)-2;

        /* make things fall fast (except balls) */
        if(w->fast && w->precip_type!=BALLS) 
                w->drops[i].precip.y-=20;

        /* make balls bounce */
        if(w->precip_type==BALLS)
        {
            w->drops[i].y_speed+=2;
            /* once they hit bottom, reverse y speed */
            if(w->drops[i].precip.y<(-GP142_YMAX+40))
            {
                w->drops[i].precip.y=-GP142_YMAX+40;
                w->drops[i].y_speed=-w->drops[i].y_speed/2;
            }
        }

        /* if drop is off to the sides or down at the bottom
        place it to the top of screen */
		if(w->drops[i].precip.y<-GP142_YMAX ||
            w->drops[i].precip.x<(-GP142_XMAX-(GP142_XMAX/4)) ||
            w->drops[i].precip.x>(GP142_XMAX+(GP142_XMAX/4)))
		{
			w->drops[i].precip.x=(rand() % (GP142_XMAX*2))-GP142_XMAX;
			w->drops[i].precip.y=GP142_YMAX;
			w->drops[i].size=RAND30(precip_size);
			w->drops[i].y_speed=RAND30(precip_yspeed)+w->drops[i].size*2;
            w->drops[i].x_speed=precip_xspeed;
		}
	}
}



void DrawRain(World *w)
{
	int		i;
	GP142_point	p1,p2;

	for(i=0;i<w->precip_qty;i++)
	{
		p1.x=w->drops[i].precip.x;
		p1.y=w->drops[i].precip.y;
		p2.x=p1.x;
		p2.y=p1.y-w->drops[i].size;
		GP142_lineP(w->precip_color,p1,p2,1);
	}
}


void DrawSnow(World *w)
{
	int		i;
	GP142_point	p1,p2;

	for(i=0;i<w->precip_qty;i++)
	{
		p1.x=w->drops[i].precip.x;
		p1.y=w->drops[i].precip.y;
		p2.x=p1.x+w->drops[i].size;
		p2.y=p1.y-w->drops[i].size;

		GP142_rectangleP(w->precip_color,p1,p2,FILL);
	}
}


#define BALL_HILITE_COLOR         WHITE

void DrawBalls(World *w)
{
	int		i;
	GP142_point	p1;

	for(i=0;i<w->precip_qty;i++)
	{
		p1.x=w->drops[i].precip.x;
		p1.y=w->drops[i].precip.y;

		GP142_circleP(w->precip_color,p1,w->drops[i].size);

        p1.x-=w->drops[i].size/2;
        p1.y+=w->drops[i].size/2;

        GP142_circleP(BALL_HILITE_COLOR,p1,w->drops[i].size/4);
	}
}




void DrawHail(World *w)
{
	int		i;
	GP142_point	p1;

	for(i=0;i<w->precip_qty;i++)
	{
		p1.x=w->drops[i].precip.x;
		p1.y=w->drops[i].precip.y;
	
		GP142_circleP(w->precip_color,p1,w->drops[i].size);
	}
}


/********************************************************************/
/* world handling functions */



World* InitWorld(void)
{
	World		*w;

	w = (World*)malloc(sizeof(World));
	if(w == NULL) BadError();


	/* intially splash screen is displayed */
	w->splash=TRUE;

	/* initialize preciptiation type */

	w->precip_type=NONE;
	w->precip_color=BLACK;
	w->precip_qty=0;

	/* drops are falling slow */
	w->fast=FALSE;

	/* initial viewpoint position and direction */
	w->pos.x=0.0;
	w->pos.y=Y_FLY;
	w->pos.z=WORLD_BOUNDS_Z+(WORLD_BOUNDS_Z/6);
	w->angle=0.0;

	w->num_objects=0;

	return w;
}

/* generate and place objects in world */

void GenerateWorld(World *w)
{
	int		i;
	int		two_thirds;
	Tree	tree;
	Point3	tp;
	

	two_thirds = 2*NUM_TREES/3;

	/* place trees in world */
	for(i=0;i<NUM_TREES;i++)
	{
		if(i<two_thirds)
		{
			tp.x=(double)((rand() % (WORLD_BOUNDS_X*2))-WORLD_BOUNDS_X);
			tp.z=(double)((rand() % (WORLD_BOUNDS_Z*2))-WORLD_BOUNDS_Z);
		} else
		{
			tp.x=(double)((rand() % (WORLD_BOUNDS_X*8))-WORLD_BOUNDS_X*4);
			tp.z=(double)((rand() % (WORLD_BOUNDS_Z*8))-WORLD_BOUNDS_Z*4);
		}
		tp.y=0.0;

		/* move trees out of the house */
		if(tp.x>-500.0 && tp.x<500.0 && tp.z>-500.0 && tp.z<500.0)
		{
			if(rand()%2)
			{
				tp.z+=1000.0;
				tp.x+=1000.0;
			}
			else
			{
				tp.z-=1000.0;
				tp.x-=1000.0;
			}
		}
	
		if(rand()%2)
			tree.color=GREEN;
		else
			tree.color=SEA_GREEN;

		tree.size = (rand()%61)+40;

		AddTreeObject(w,&tree,&tp);
	}

	GenerateHouse(w);

	/* set initial view */
	SetView(w,0.0,0.0);

	/* generate ground texture */
	GenerateTexture(w);
}


/* generate random rectangles in the texture array
the of the pixels can be any valid GP142 color or
TRANSP_COLOR if they are transparent

  This texture will be mapped on the ground when
  the DrawScene() is called.

  It is possible that this texture be read from
  a file given that the colors are converted to
  GP142 colors.
*/

#define TEX_NUM_RECTS				15
#define TEX_MAX_RECT_SIZE			21

void GenerateTexture(World *w)
{
	int		i,k,j;
	int		x,y;
	int		sx,sy;
	int		color;

	memset(w->texture,TRANSP_COLOR,TEX_SIZE*TEX_SIZE);

	/* place TEX_NUM_RECTS randomly shaped rectangles */
	for(j=0;j<TEX_NUM_RECTS;j++)
	{
		x = (rand() % (TEX_SIZE-1));
		y = (rand() % (TEX_SIZE-1));

		sx = (rand() % (TEX_MAX_RECT_SIZE-5))+5;
		sy = (rand() % (TEX_MAX_RECT_SIZE-1))+1;

		switch(rand() % 3)
		{
		case 0:
			color=YELLOW;
			break;

		case 1:
			color=BROWN;
			break;

		default:
			color = BLACK;
			break;
		}

		for(i=x;i<sx+x;i++)
		{
			for(k=y;k<sy+y;k++)
			{
				if(k<TEX_SIZE && i<TEX_SIZE)
					w->texture[k+(i*TEX_SIZE)]=color;
			}
		}
	}

}

/* generate a house shaped triangle object 
Note: if the points on the triangles have to go clockwise
for correct hidden surface removal
If a surface contains a door for example, it needs to be
divided up into rectangles (or triangles) so that the door
is one rectangle and the walls around it may be 3 or 4.*/
void GenerateHouse(World *w)
{
	TriangleList	tri_list;
	Point3			pos;
	Point3			b1,b2,b3,b4;  /* base points */
	Point3			r1,r2,r3,r4;	/* roof points */
	Point3			w1,w2,w3,w4;	/* side window points */
	Point3			f1,f2,f3,f4;	/* front window points */
	Point3			d1,d2,d3,d4;	/* door points */
	Point3			t1,t2;			/* top of the roof points*/
	Point3			tmp1,tmp2;

	InitTriList(&tri_list);

	b1.x=100.0; b1.y=0.0; b1.z=-200.0;
	b2.x=100.0; b2.y=0.0; b2.z=200.0;
	b3.x=-100.0; b3.y=0.0; b3.z=200.0;
	b4.x=-100.0; b4.y=0.0; b4.z=-200.0;

	r1.x=100.0; r1.y=200.0; r1.z=-200.0;
	r2.x=100.0; r2.y=200.0; r2.z=200.0;
	r3.x=-100.0; r3.y=200.0; r3.z=200.0;
	r4.x=-100.0; r4.y=200.0; r4.z=-200.0;

	w1.x=100.0; w1.y=50.0; w1.z=-100.0;
	w2.x=100.0; w2.y=50.0; w2.z=0.0;
	w3.x=100.0; w3.y=150.0; w3.z=0.0;
	w4.x=100.0; w4.y=150.0; w4.z=-100.0;

	f1.x=25.0; f1.y=80.0; f1.z=-200.0;
	f2.x=75.0; f2.y=80.0; f2.z=-200.0;
	f3.x=75.0; f3.y=120.0; f3.z=-200.0;
	f4.x=25.0; f4.y=120.0; f4.z=-200.0;

	d1.x=-60.0; d1.y=0.0; d1.z=-200.0;
	d2.x=0.0; d2.y=0.0; d2.z=-200.0;
	d3.x=0.0; d3.y=120.0; d3.z=-200.0;
	d4.x=-60.0; d4.y=120.0; d4.z=-200.0;

	t1.x=0.0; t1.y=300.0; t1.z=-200.0;
	t2.x=0.0; t2.y=300.0; t2.z=200.0;


	/* front wall (door and window) */
	AddRectangle(&tri_list,GOLD,&d4,&d3,&d2,&d1);
	AddRectangle(&tri_list,BLUE,&f4,&f3,&f2,&f1);

	tmp1.x=r4.x; tmp1.y=d4.y; tmp1.z=b1.z;
	tmp2.x=r1.x; tmp2.y=d4.y; tmp2.z=b1.z;
	AddRectangle(&tri_list,BROWN,&tmp1,&r4,&r1,&tmp2);

	AddRectangle(&tri_list,BROWN,&tmp1,&d4,&d1,&b4);
	tmp1.x=f1.x; tmp1.y=b1.y;
	AddRectangle(&tri_list,BROWN,&d3,&f4,&tmp1,&d2);

	tmp1.x=f2.x; tmp2.x=r1.x; tmp2.y=f3.y;
	AddRectangle(&tri_list,BROWN,&f3,&tmp2,&b1,&tmp1);

	tmp1.x=f1.x; tmp2.x=f2.x; tmp2.y=b1.y;
	AddRectangle(&tri_list,BROWN,&f1,&f2,&tmp2,&tmp1);

	/* right side wall + window */
	AddRectangle(&tri_list,BLUE,&w4,&w3,&w2,&w1);

	tmp1.x=w4.x; tmp1.y=r1.y; tmp1.z=w4.z;
	tmp2.x=w1.x; tmp2.y=b1.y; tmp2.z=w4.z;
	AddRectangle(&tri_list,MED_GRAY,&r1,&tmp1,&tmp2,&b1);

	tmp1.z=w3.z; tmp2.z=w3.z;
	AddRectangle(&tri_list,MED_GRAY,&tmp1,&r2,&b2,&tmp2);

	tmp1.y=b1.y; tmp1.z=w1.z;
	AddRectangle(&tri_list,MED_GRAY,&tmp1,&w1,&w2,&tmp2);

	tmp1.y=r1.y; tmp2.y=r1.y;
	AddRectangle(&tri_list,MED_GRAY,&tmp1,&tmp2,&w3,&w4);

	/* back wall */
	AddRectangle(&tri_list,BROWN,&b2,&r2,&r3,&b3);
	/* left side wall */
	AddRectangle(&tri_list,MED_GRAY,&b3,&r3,&r4,&b4);

	/* front roof top */
	AddTriangle(&tri_list,ORANGE,&r4,&t1,&r1);
	/* right roof */
	AddRectangle(&tri_list,BLACK,&t1,&t2,&r2,&r1);
	/* back roof top */
	AddTriangle(&tri_list,ORANGE,&r2,&t2,&r3);
	/* left roof */
	AddRectangle(&tri_list,BLACK,&r3,&t2,&t1,&r4);


	pos.x=-200.0;
	pos.y=0.0;
	pos.z=0.0;
	AddTriObject(w,&tri_list,&pos);



	/* add a bench */
	InitTriList(&tri_list);

	/* base */
	b1.x=20.0; b1.y=0.0; b1.z=-100.0;
	b2.x=20.0; b2.y=0.0; b2.z=50.0;
	b3.x=-20.0; b3.y=0.0; b3.z=50.0;
	b4.x=-20.0; b4.y=0.0; b4.z=-100.0;

	/* top */
	r1.x=20.0; r1.y=20.0; r1.z=-100.0;
	r2.x=20.0; r2.y=20.0; r2.z=50.0;
	r3.x=-20.0; r3.y=20.0; r3.z=50.0;
	r4.x=-20.0; r4.y=20.0; r4.z=-100.0;

	/* sides */
	AddRectangle(&tri_list,BROWN,&b4,&r4,&r1,&b1);
	AddRectangle(&tri_list,BROWN,&b1,&r1,&r2,&b2);
	AddRectangle(&tri_list,BROWN,&b2,&r2,&r3,&b3);
	AddRectangle(&tri_list,BROWN,&b3,&r3,&r4,&b4);

	/* and top*/
	AddRectangle(&tri_list,LT_GRAY,&r4,&r3,&r2,&r1);


	pos.x=0.0;
	pos.y=0.0;
	pos.z=0.0;
	AddTriObject(w,&tri_list,&pos);
}




/* free up memory used by World structure */

void CloseWorld(World *w)
{
	int		i;

	for(i=0;i<w->num_objects;i++)
	{
		if(w->obj[i].obj_type == TRIANGLE_OBJ)
			free((void*)w->obj[i].tri_list);
	}

	free((void*)w);
}


/* add a tree to the object list, its parameters and 
position are passed in tree, and pos respectively*/
void AddTreeObject(World *w, Tree *tree, Point3 *pos)
{
	if(w->num_objects<MAX_OBJECTS)
	{
		w->obj[w->num_objects].obj_type = TREE_OBJ;
		memcpy((void*)&w->obj[w->num_objects].o,(void*)pos,sizeof(Point3));
		memcpy((void*)&w->obj[w->num_objects].tree,(void*)tree,sizeof(Tree));
		w->num_objects++;
	}
}


/* add a triangle list to the object list, again its
parameters and position are input parameters */
void AddTriObject(World *w, TriangleList *tri_list, Point3 *pos)
{
	int		i,k;

	if(w->num_objects<MAX_OBJECTS)
	{
		w->obj[w->num_objects].obj_type = TRIANGLE_OBJ;
		memcpy((void*)&w->obj[w->num_objects].o,(void*)pos,sizeof(Point3));

		/* place object at given position in world */
		for(i=0;i<tri_list->num_triangles;i++)
		{
			for(k=0;k<TRI_EDGES;k++)
			{
				tri_list->tri[i].e[k].x+=pos->x;
				tri_list->tri[i].e[k].y+=pos->y;
				tri_list->tri[i].e[k].z+=pos->z;
			}
		}

		w->obj[w->num_objects].tri_list = (TriangleList*)malloc(sizeof(TriangleList));
		if(w->obj[w->num_objects].tri_list == NULL) BadError();

		memcpy((void*)w->obj[w->num_objects].tri_list,
			(void*)tri_list,sizeof(TriangleList));
		w->num_objects++;
	}
}

/* initialize triangle list */
void InitTriList(TriangleList *tri_list)
{
	tri_list->num_triangles=0;
}

/* add triangle to triangle list */
void AddTriangle(TriangleList *tri_list, int color,
				 Point3 *e1, Point3 *e2, Point3 *e3)
{
	Triangle		tri;

	if(tri_list->num_triangles<MAX_TRIANGLES)
	{
		tri.color = color;
		memcpy((void*)&tri.e[TRI_E1],e1,sizeof(Point3));
		memcpy((void*)&tri.e[TRI_E2],e2,sizeof(Point3));
		memcpy((void*)&tri.e[TRI_E3],e3,sizeof(Point3));

		memcpy((void*)&tri_list->tri[tri_list->num_triangles],
				(void*)&tri, sizeof(Triangle));

		tri_list->num_triangles++;
	}
}


/* add a rectangle to the triangle list (in fact
2 triangles) */
void AddRectangle(TriangleList *tri_list, int color,
				  Point3 *e1, Point3 *e2, Point3 *e3, Point3 *e4)
{
	AddTriangle(tri_list,color,e1,e2,e3);
	AddTriangle(tri_list,color,e3,e4,e1);
}




/* set view point poisition and angle. This is done by supplying
a pos_inc value which is the magnitude of the position vector, and
an angle_inc for direction */

void SetView(World *w, double pos_inc, double angle_inc)
{
	int		i,k,l;
	double	x,y,z;	
	double	tcos,tsin;

	w->angle += angle_inc;

	tsin = sin(w->angle);
	tcos = cos(w->angle);

	/* break down pos_inc into components */
	w->pos.x += pos_inc * tsin;
	w->pos.z += pos_inc * tcos;

	/* wrap angle around 0->PI / -PI->0 */
	if(w->angle>PI) w->angle-=PI*2;
	if(w->angle<-PI) w->angle+=PI*2;

	for(i=0;i<w->num_objects;i++)
	{
		/* translate points */
		x = w->obj[i].o.x + w->pos.x;
		y = w->obj[i].o.y + w->pos.y;
		z = w->obj[i].o.z + w->pos.z;

		/* rotate points */
		w->obj[i].to.x = (x * tcos) - (z * tsin);
		w->obj[i].to.y = y;
		w->obj[i].to.z = (x * tsin) + (z * tcos);

		/* a triangle objects needs to have all the 
		triangles inside of it translated and rotated */
		if(w->obj[i].obj_type == TRIANGLE_OBJ)
		{
			for(k=0;k<w->obj[i].tri_list->num_triangles;k++)
			{
				for(l=0;l<TRI_EDGES;l++)
				{
					/* translate triangle edges */
					x = w->obj[i].tri_list->tri[k].e[l].x + w->pos.x;
					y = w->obj[i].tri_list->tri[k].e[l].y + w->pos.y;
					z = w->obj[i].tri_list->tri[k].e[l].z + w->pos.z;

					/* rotate triangle edges */
					w->obj[i].tri_list->tri[k].te[l].x = (x * tcos) - (z * tsin);
					w->obj[i].tri_list->tri[k].te[l].y = y;
					w->obj[i].tri_list->tri[k].te[l].z = (x * tsin) + (z * tcos);
				}
			}
		}
	}
}


/* 
	Project a 3d point from world space to screen space
	also perform clipping on the z coordinate so that
	anything behind the viewer or relly far will not
	show.
	There is also clipping for stuff that is outside
	of visible window (-GP142_XMAX to GP142_XMAX).

	The mulz value is used as a scaling factor when
	drawing objects (trees).

  perspective projection:
	screen x = world x * view distance / world z
	screen y = world y * view distance / world z
*/

#define Z_CLIP_CLOSE				20.0
#define Z_CLIP_FAR					9000.0

int ProjectPt(Point3 *pt, GP142_point *sp, double *mulz)
{
	double		mz;

	/* visible ? */
	if(pt->z>Z_CLIP_CLOSE && pt->z<Z_CLIP_FAR)
	{
		mz = VIEWING_DISTANCE / pt->z;

		sp->x = (int)(pt->x * mz);
		sp->y = (int)(pt->y * mz);

		*mulz = mz;

		return TRUE;
	} else
		return FALSE;
}



/* check if triangle is facing the viewer, if not, don't draw
surface. (hidden surface removel using backface culling) */
int IsTriangleVisible(Point3 *e1, Point3 *e2, Point3 *e3)
{
	Point3		normal;
	Point3		u,v;
	double		visible;

	/* build vectors u and v from the edges of the triangle */
	u.x = e3->x - e1->x;
	u.y = e3->y - e1->y;
	u.z = e3->z - e1->z;

	v.x = e2->x - e1->x;
	v.y = e2->y - e1->y;
	v.z = e2->z - e1->z;

	/* perform cross product (u x v) to find normal to plane */
	normal.x = u.y*v.z - u.z*v.y;
	normal.y = u.x*v.z - u.z*v.x;
	normal.z = u.x*v.y - u.y*v.x;

	/* calculate dot product (normal . e1) to figure out the
	cos of the angle between vectors. Since the cos switches
	sign at pi/2, checking the sign of the result is sufficient.
	The 2 vectors checked (normal and viewer position) detemine
	if this surface is to be visible or not*/
	visible = normal.x*e1->x + normal.y*(-e1->y) + normal.z*e1->z;

	if(visible>0.0)
		return TRUE;
	else
		return FALSE;

}


/* check for at least one edge of the triangle to be on screen
(only x axis is checked) */
int IsTriangleOnScreen(GP142_point *p1, GP142_point *p2, 
					   GP142_point *p3)
{
	if(p1->x>GP142_XMAX && p2->x>GP142_XMAX && p3->x>GP142_XMAX)
		return FALSE;

	if(p1->x<-GP142_XMAX && p2->x<-GP142_XMAX && p3->x<-GP142_XMAX)
		return FALSE;

	return TRUE;
}


/* determine drawing order of triangles 
This function is a way to implement additional hidden surface 
removal of stacked objects (in the y direction). If the 
triangle's y extent crosses the origin, draw triangle last 
(return TRUE). If triangle has either all negative or all
positive edges, return FALSE.

Ex: a table with a top and 4 legs. If it is all below origin
(all y's are < 0) draw legs and then top (thats how the object
was loaded). If the table moves up (or viewer moves down)
so that the legs will be crossing the origin, then draw the
top first and then the legs. The table will not move up
so that all points will be above origin, this would mean
that the viewer is moving underground.*/

int IsTriangleDrawnLast(Point3 *e1, Point3 *e2, Point3 *e3)
{
	/* all negative */
	if(e1->y<0.0 && e2->y<0.0 && e3->y<0.0) return FALSE;

	/* all positive */
	if(e1->y>0.0 && e2->y>0.0 && e3->y>0.0) return FALSE;

	return FALSE;
}



/* draw everything */

void DrawScene(World *w)
{
	int				i;
	GP142_point		sp;
	double			mulz;

	DrawBackground(w);

	DrawGround(w);

	SortObjects(w);

	for(i=0;i<w->num_objects;i++)
	{
		if(w->obj[i].obj_type==TREE_OBJ)
		{
			if(ProjectPt(&w->obj[i].to,&sp,&mulz))
			{
				DrawTree(&w->obj[i].tree,&sp,mulz);
			}
		}

		if(w->obj[i].obj_type==TRIANGLE_OBJ)
		{
			DrawTriangleObject(w->obj[i].tri_list);
		}
	}
}



/* draw triangle object:
draw all triangles part of a triangle object
*/

void DrawTriangleObject(TriangleList *tri_list)
{
	Point3		*e1,*e2,*e3;
	int			i,last_cnt;

	/* number of triangles to be drawn last */
	last_cnt = 0;

	/* draw triangles */
	for(i=0;i<tri_list->num_triangles;i++)
	{
		/* get point addresses (this is so that 
		memcpy is avoided) */
		e1=&tri_list->tri[i].te[TRI_E1];
		e2=&tri_list->tri[i].te[TRI_E2];
		e3=&tri_list->tri[i].te[TRI_E3];
		
		/* is triangle hidden from viewer? */
		if(IsTriangleVisible(e1,e2,e3))
		{
			if(IsTriangleDrawnLast(e1,e2,e3))
			{
				/* mark triangle to be drawn last */
				tri_list->sort_tri[last_cnt++]=&tri_list->tri[i];
			} else
			{
				/* draw triangle */
				DrawTriangle(tri_list->tri[i].color,e1,e2,e3);
			}
		}
	}

	/* draw triangles marked last */
	for(i=0;i<last_cnt;i++)
	{
		/* draw triangle */
		DrawTriangle(tri_list->sort_tri[i]->color,
					&tri_list->sort_tri[i]->te[TRI_E1],
					&tri_list->sort_tri[i]->te[TRI_E2],
					&tri_list->sort_tri[i]->te[TRI_E3]
					);
	}
}

/* project and draw triangle */
void DrawTriangle(int color, Point3 *e1, Point3 *e2, Point3 *e3)
{
	GP142_point	sp1,sp2,sp3;
	double		mulz;

	/* if any points of the triangle are clipped, don't draw it*/
	if(ProjectPt(e1,&sp1,&mulz) && ProjectPt(e2,&sp2,&mulz) &&
				ProjectPt(e3,&sp3,&mulz))
	{
		/* is triangle even on screen? */
		if(IsTriangleOnScreen(&sp1,&sp2,&sp3))
		{
			GP142_triangleP(color,sp1,sp2,sp3,FILL);
		}
	}
}


/* draw background: sun, sky, ground and mountains */

void DrawBackground(World *w)
{
	GP142_point			p1,p2,p3;

	/* ground */
	p1.x=-GP142_XMAX;
	p1.y=0;
	p2.x=GP142_XMAX;
	p2.y=-GP142_YMAX;

	GP142_rectangleP(FOREST_GREEN,p1,p2,FILL);

	/* sky */
	p1.y=GP142_YMAX;
	p2.y=0;

	GP142_rectangleP(ICE_BLUE,p1,p2,FILL);

	/* upper sky (darker) */
	p2.y=2*GP142_YMAX/3;
	GP142_rectangleP(CYAN,p1,p2,FILL);

	/* figure out correct position of sun + mountains */
	
	p1.x=(int)(-w->angle*VIEWING_DISTANCE);
	p1.y=3*GP142_YMAX/4;

	/* sun */
	GP142_circleP(YELLOW,p1,30);

	p2.x=p1.x-GP142_XMAX;
	p2.y=p1.y;
	p3.x=p1.x-(GP142_XMAX/2);
	p3.y=p1.y-GP142_YMAX/8;
	GP142_triangleP(WHITE,p1,p2,p3,FILL);

	/* various mountains */

	p1.x+=GP142_XMAX;
	p2.x=p1.x-(2*GP142_XMAX/3);
	p2.y=0;
	p3.x=p1.x+GP142_XMAX/2;
	p3.y=0;

	GP142_triangleP(DUSTY_PLUM,p1,p2,p3,FILL);

	p1.y-=GP142_YMAX/5;
	p1.x=p1.x-(7*GP142_XMAX/5);
	p2.x=p1.x-GP142_XMAX/2;
	p3.x=p1.x+GP142_XMAX;

	GP142_triangleP(OLIVE,p2,p3,p1,FILL);

	p1.y=GP142_YMAX/2;
	p1.x=p1.x+GP142_XMAX*4;
	p2.x=p1.x-GP142_XMAX;
	p3.x=p1.x+GP142_XMAX*3;

	GP142_triangleP(GOLD,p1,p2,p3,FILL);

	p1.y=GP142_YMAX/2;
	p1.x=p1.x-GP142_XMAX*9;
	p2.x=p1.x-GP142_XMAX*3;
	p3.x=p1.x+GP142_XMAX*4;

	GP142_triangleP(DUSTY_PLUM,p1,p2,p3,FILL);

	p1.y=GP142_YMAX;
	p1.x=p1.x+GP142_XMAX*12;
	p2.x=p1.x-GP142_XMAX;
	p3.x=p1.x+GP142_XMAX/2;

	GP142_triangleP(NAVY_BLUE,p1,p2,p3,FILL);

	p2.x+=(int)(7.85*(double)GP142_YMAX/6.0);
	p3.x-=(int)(4.9*(double)GP142_YMAX/6.0);
	p2.y=5*GP142_YMAX/6;
	p3.y=5*GP142_YMAX/6;

	GP142_triangleP(WHITE,p1,p2,p3,FILL);

}


/* sort the object array by ascending z coordinate.
this way further objects (higher z) are drawn first */

void SortObjects(World *w)
{
	int		i,k;
	Object	obj;

	for(i=0;i<w->num_objects;i++)
	{
		for(k=i+1;k<w->num_objects;k++)
		{
			if(w->obj[i].to.z < w->obj[k].to.z)
			{
				memcpy((void*)&obj,(void*)&w->obj[i],sizeof(Object));
				memcpy((void*)&w->obj[i],(void*)&w->obj[k],sizeof(Object));
				memcpy((void*)&w->obj[k],(void*)&obj,sizeof(Object));
			}
		}
	}
}


/* draw a single tree; its size is determined by the
mulz factor which scales it according to distance*/

#define TREE_H_SIZE		50.0
#define TREE_V_SIZE		200.0

void DrawTree(Tree *tree, GP142_point *sp, double mulz)
{
	GP142_point		p1,p2,p3;
	int				edge_clip;

	/* draw tree only if on screen */
	edge_clip = GP142_XMAX + (int)(TREE_H_SIZE * mulz);
	if(sp->x>-edge_clip && sp->x<edge_clip)
	{
		p1.x=sp->x - (int)(TREE_H_SIZE * mulz);
		p1.y=sp->y + (int)((double)tree->size * mulz);
		p2.x=sp->x + (int)(TREE_H_SIZE * mulz);
		p2.y=sp->y + (int)((double)tree->size * mulz);
		p3.x=sp->x;
		p3.y=sp->y + (int)((TREE_V_SIZE+(double)tree->size) * mulz);

		GP142_triangleP(tree->color,p1,p2,p3,FILL);

		p1.x=sp->x - (int)(8.0 * mulz);
		p1.y=sp->y + (int)((double)tree->size * mulz);
		p2.x=sp->x + (int)(8.0 * mulz);
		p2.y=sp->y;

		GP142_rectangleP(BROWN,p1,p2,FILL);
	}

}

/* map the w->texture on the lower half of the screen.


	given these formulas:
		screen x = world x * view distance / world z
		screen y = world y * view distance / world z

    we derive the folowing:

	1.	world x = ( screen x * world z ) / view distance
	2.	world y = ( screen y * world z ) / view distance

	3.	world z = ( world x * view distance ) / screen x
	4.	world z = ( world y * view distance ) / screen y

   Since this function maps a texture on the ground, we know
   that world y is constant throughout the whole operation.
	This way we can get world z from (4.) by substituting
	screen y. From (1.) we calculate world x.
	For every screen coordinate we
	have a corresponding world x and world z. Since world x
	and world z are the texture coordinates, we can map the
	texture on the screen.

	float was used to speed up calculations
*/

void DrawGround(World *w)
{
	GP142_point	pixel;
	float		tsin,tcos;
	float		x,z,xr,zr;  /* world points, rotated world points */
	float		posx,posz;	/* view point position */
	int			tx,tz;		/* actual texture coordinates */
	int			sx,sy;		/* current screen coordinates */
	char		color;		/* pixel color */
	float		div_view_d;	/* 1.0 / view distance */
	
	tsin = (float)-sin(w->angle);
	tcos = (float)cos(w->angle);

	posx = (float)w->pos.x;
	posz = (float)w->pos.z;

	div_view_d = 1.0f / (float)VIEWING_DISTANCE;

	/* every other pixel is skipped */

	/* for each screen line (going y direction)*/
	for(sy=-10;sy>-GP142_YMAX+INFO_BAR_HEIGHT;sy-=2)
	{
        /* texture z*/
		z = ((float)VIEWING_DISTANCE*(float)w->pos.y)/(float)sy;
		
		/* for each pixel on the line (going x direction)*/
		for(sx=-GP142_XMAX;sx<=GP142_XMAX;sx+=2)
		{
            /* figure out texture x */
			x = (float)sx * z * div_view_d;

			/* x, y contain the world coordinates
			derived from sx and sy */

			/* rotate */
			xr = x*tcos - z*tsin;
			zr = x*tsin + z*tcos;

			/* translate */
			xr -= posx;
			zr -= posz;

			/* to get texture coordinates (tx,tz) mask
			xr and zr so that they always wrap from 0
			to TEX_SIZE-1 (NOTE: TEX_SIZE must be
            a power of 2)*/
			tx = (int)(xr) & (TEX_SIZE-1);
			tz = (int)(zr) & (TEX_SIZE-1);
			
			/* get pixel color from texture */
			color=w->texture[tx+(tz*TEX_SIZE)];

			pixel.x = sx;
			pixel.y = sy;

			/* if not transparent, draw pixel */
			if(color!=TRANSP_COLOR) GP142_pixelP((int)color,pixel);	
		}
	}
}


/*******************************************************************/

/* button handling functions */

void ButtonInit(ButtonList *bl)
{
	bl->num_buttons=0;
}

/* add a button to the button list */

void AddButton(ButtonList *bl, GP142_point *ll_pos, char *title, int id)
{
	if(bl->num_buttons<MAX_BUTTONS)
	{
		bl->buttons[bl->num_buttons].button_id=id;
		bl->buttons[bl->num_buttons].ll_pos.x=ll_pos->x;
		bl->buttons[bl->num_buttons].ll_pos.y=ll_pos->y;
		strcpy(bl->buttons[bl->num_buttons].title,title);
		bl->buttons[bl->num_buttons].hilited=FALSE;

		bl->num_buttons++;
	}
}

/* check if there is a button at the mouse
if so, return the button id, otherwise return NO_BUTTON_HIT. */

int ButtonHit(ButtonList *bl, GP142_point *mouse)
{
	int		i;
	int		bx,by;

	for(i=0;i<bl->num_buttons;i++)
	{
		bx = bl->buttons[i].ll_pos.x;
		by = bl->buttons[i].ll_pos.y;

		if(mouse->x>bx && mouse->x<(bx+BUTTON_WIDTH) &&
			mouse->y>by && mouse->y<(by+BUTTON_HEIGHT))

			return bl->buttons[i].button_id;
	}

	return NO_BUTTON_HIT;
}


/* set hilite status of a button, if hilite is TRUE
then it is hlited */
void SetButtonHilite(ButtonList *bl, int id, int hilite)
{
	int		i;

	for(i=0;i<bl->num_buttons;i++)
	{
		if(bl->buttons[i].button_id==id)
		{
			bl->buttons[i].hilited=hilite;
			break;
		}
	}
}


/* draw all buttons in the button list, and hilite the one
with the hilite_id */

#define BUTTON_BORDER		WHITE
#define BUTTON_TEXT_SIZE	14

void DrawButtons(ButtonList *bl)
{
	int				i;
	GP142_point		p1,p2;
	int				fill_color,text_color;
	

	for(i=0;i<bl->num_buttons;i++)
	{
		if(bl->buttons[i].hilited)
		{
			fill_color = NAVY_BLUE;
			text_color = WHITE;
		} else
		{
			fill_color = BLUE;
			text_color = WHITE;
		}

		p1.x=bl->buttons[i].ll_pos.x;
		p1.y=bl->buttons[i].ll_pos.y;
		p2.x=p1.x+BUTTON_WIDTH;
		p2.y=p1.y+BUTTON_HEIGHT;

		/* button "filling" */
		GP142_rectangleP(fill_color,p1,p2,FILL);

		p2.x++;
		p1.y--;

		/* lower right shadonw */
		GP142_rectangleP(MED_GRAY,p1,p2,1);

		p2.x--;
		p1.y++;

		/* border */
		GP142_rectangleP(BUTTON_BORDER,p1,p2,1);


		p1.x+=5;
		p1.y+=(BUTTON_HEIGHT/2)-(BUTTON_TEXT_SIZE/2);

		/* title */
		GP142_printfP(text_color,p1,BUTTON_TEXT_SIZE,"%s",
			bl->buttons[i].title);
	}
}






/********************************************************************/

/* misc functions */

/* print failure error message */
void BadError(void)
{
	GP142_event_t		event_t;
	int					event;
	GP142_point			p1;

	GP142_clear();

	p1.x=-110;
	p1.y=0;
	GP142_printfP(RED,p1,20,"ALLOCATION FAILURE");

	while(1)
	{
		event = GP142_await_eventP(&event_t);

		if(event == GP142_QUIT) exit(EXIT_FAILURE);
	}
}

